/*
  Copyright (c) 2014 by I.Kato @ BS21 Lab
  SRAM512K Shield (BS21LAB-018) library for Arduino
  
  Based: 
    Copyright (c) 2010 by arms22 (arms22 at gmail.com)
    Microchip 23x256 SPI SRAM library for Arduino
  
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
*/

#ifndef SPISRAM512K_H
#define SPISRAM512K_H

#include <SPI.h>

// SRAM 512K Shield (BS21LAB-018) 用ライブラリ
// ※ 本ライブラリは、arms22氏作成の SPISRAM ライブラリをベースにしています。

// SRAM の容量が 512kバイトの為、指定できるアドレスは 19bit長になります。

class SPISRAM512K
{
public:
	struct MemCell {
		SPISRAM512K & device;
		unsigned long address;
		MemCell & operator= (byte data) {
			device.write (address, data);
			return *this;
		}
		operator byte() {
			return device.read (address);
		}
		MemCell (SPISRAM512K &d, unsigned long a) : device(d), address(a) {};
		~MemCell() {};
	};
	MemCell operator[] (unsigned long address) {
		MemCell memcell (*this, address);
		return memcell;
	}
public:
	SPISRAM512K (byte ncsPin);
	void init (void);
public:
	void setSpiConditionForSRAM512K (void);  // SPI 設定を SRAM512K shield 用(2MHz)にする
	void setSpiConditionForDefault  (void);  // SPI 設定を Arduino IDE の default 設定(4MHz)にする
public:
	byte read (unsigned long address);
	void read (unsigned long address, byte *buffer, unsigned int size);
	void write(unsigned long address, byte data);
	void write(unsigned long address, byte *buffer, unsigned int size);
private:
	byte _ncsPin;
	byte _buf[4];
	void select   (void);
	void deselect (void);
	byte transfer (byte length);
};

#endif
