/*
  Copyright (c) 2014 by I.Kato @ BS21 Lab
  
  This file is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
*/

#include <SPI.h>
#include <SPISRAM512K.h>

SPISRAM512K myRAM(10); // CS pin

word buffer1[256];
word buffer2[256];

void setup()
{
  Serial.begin(9600);

  while (!Serial) {
    ; // wait for serial port to connect. Needed for Leonardo only
  }

  myRAM.init(); // SRAM512K との SPI通信初期化

  // SRAM512K の全メモリの読み書きテスト
  // 512キロバイトの読み書き(トータル1MB)に、約12秒かかります。
  // よって、転送速度は約85kB/秒程度になります。
  Serial.println("block write & read test:");

  unsigned long addr = 0x00000; // 0x00000～0x7FFFF の範囲
  int hi, lo;

  for (hi = 0; hi < (int)(512 * 1024L / 512); hi++) { // 1024回のループ
    Serial.print  ("turn ");
    Serial.println(hi);

    // buffer1 にデータを書き込む
    for (lo = 0; lo < 256; lo++) {
      buffer1[lo] = (word)((addr + lo + lo) >> 3);  // 0x0000～0xFFFF のいずれか
    }

    // SRAM512K へ連続 512バイト書き込み
    myRAM.write (addr, (byte*)buffer1, sizeof(buffer1));

    // SRAM512K から連続 512バイト読み込み
    myRAM.read (addr, (byte*)buffer2, sizeof(buffer2));

    // buffer1 と buffer2 を比較する
    for (lo = 0; lo < 256; lo++) {
      if (buffer1[lo] != buffer2[lo]) { // 書き込んだ値と読み出した値が違っている
        Serial.print  ("err: 0x");
        Serial.print  (buffer1[lo], HEX);
        Serial.print  (" != 0x");
        Serial.println(buffer2[lo], HEX);
      }
    }

    addr += 512;  // 次のアドレス
  }

  Serial.println("done."); // 終了
}

void loop()
{
}
