/*
  Copyright (c) 2014 by I.Kato @ BS21 Lab
  
  Based: 
    Copyright (c) 2010 by arms22 (arms22 at gmail.com)
  
  This file is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
*/

#include <SPI.h>
#include <SPISRAM512K.h>

SPISRAM512K myRAM(10); // CS pin

char buffer[128];

void setup()
{
  Serial.begin(9600);

  while (!Serial) {
    ; // wait for serial port to connect. Needed for Leonardo only
  }

  myRAM.init(); // SRAM512K との SPI通信初期化

  Serial.println("Byte write...");
  myRAM.write(0,'H');
  myRAM.write(1,'e');
  myRAM.write(2,'l');
  myRAM.write(3,'l');
  myRAM.write(4,'o');
  myRAM.write(5,'-');

  Serial.println("Byte read...");
  Serial.println( (char)myRAM.read(0) );
  Serial.println( (char)myRAM.read(1) );
  Serial.println( (char)myRAM.read(2) );
  Serial.println( (char)myRAM.read(3) );
  Serial.println( (char)myRAM.read(4) );
  Serial.println( (char)myRAM.read(5) );

  Serial.println("\nByte write...");
  myRAM.write(0x7FFFC,'W');
  myRAM.write(0x7FFFD,'o');
  myRAM.write(0x7FFFE,'r');
  myRAM.write(0x7FFFF,'l');
  myRAM.write(0x80000,'d'); // 存在しないメモリに書き込む
  myRAM.write(0x80001,'!'); // 同上
  myRAM.write(0x80002,'!'); // 同上

  Serial.println("Byte read...");
  Serial.println( (char)myRAM.read(0x7FFFC) );
  Serial.println( (char)myRAM.read(0x7FFFD) );
  Serial.println( (char)myRAM.read(0x7FFFE) );
  Serial.println( (char)myRAM.read(0x7FFFF) );
  Serial.println( (char)myRAM.read(0x80000) ); // 存在しないメモリなので読み込みに失敗する
  Serial.println( (char)myRAM.read(0x80001) ); // 同上
  Serial.println( (char)myRAM.read(0x80002) ); // 同上

  Serial.println("\nseq write...");
  unsigned long addr = 0x7FF00;
  myRAM.write(addr, (byte*)"Hello world!!", sizeof("Hello world!!"));

  Serial.println("seq read...");
  myRAM.read(addr, (byte*)buffer, sizeof(buffer));
  Serial.println( buffer );

  Serial.println("\nByte read at operator[]...");
  myRAM[100] = 'H';
  myRAM[101] = 'e';
  myRAM[102] = 'l';
  myRAM[103] = 'l';
  myRAM[104] = 'o';
  Serial.println( (char)myRAM[100] );
  Serial.println( (char)myRAM[101] );
  Serial.println( (char)myRAM[102] );
  Serial.println( (char)myRAM[103] );
  Serial.println( (char)myRAM[104] );

  Serial.println("\nRandom read/write...");
  for(int i = 0; i < 256; i++){
    unsigned long addr = random() & 0x7FFFF;
    byte val = i;
    Serial.print( addr, HEX );
    Serial.print( " " );
    Serial.print( val, HEX );
    myRAM[addr] = val;
    Serial.print( " " );
    Serial.println( myRAM[addr], HEX );
  }

  Serial.println("\ndone.");
}

void loop()
{
}
